@@ -86,6 +86,7 @@ class SdoClient(SdoBase):
             except SdoCommunicationError as e:
                 retries_left -= 1
                 if not retries_left:
+                    self.abort(0x5040000) 
                     raise
                 logger.warning(str(e))
 
@@ -98,16 +99,15 @@ class SdoClient(SdoBase):
         self.send_request(request)
         logger.error("Transfer aborted by client with code 0x{:08X}".format(abort_code))
 
-    def upload(self, index, subindex):
+    def upload(self, index: int, subindex: int) -> bytes:
         """May be called to make a read operation without an Object Dictionary.
 
-        :param int index:
+        :param index:
             Index of object to read.
-        :param int subindex:
+        :param subindex:
             Sub-index of object to read.
 
         :return: A data object.
-        :rtype: bytes
 
         :raises canopen.SdoCommunicationError:
             On unexpected response or timeout.
@@ -132,16 +132,22 @@ class SdoClient(SdoBase):
                     data = data[0:size]
         return data
 
-    def download(self, index, subindex, data, force_segment=False):
+    def download(
+        self,
+        index: int,
+        subindex: int,
+        data: bytes,
+        force_segment: bool = False,
+    ) -> None:
         """May be called to make a write operation without an Object Dictionary.
 
-        :param int index:
+        :param index:
             Index of object to write.
-        :param int subindex:
+        :param subindex:
             Sub-index of object to write.
-        :param bytes data:
+        :param data:
             Data to be written.
-        :param bool force_segment:
+        :param force_segment:
             Force use of segmented transfer regardless of data size.
 
         :raises canopen.SdoCommunicationError:
@@ -155,7 +161,7 @@ class SdoClient(SdoBase):
         fp.close()
 
     def open(self, index, subindex=0, mode="rb", encoding="ascii",
-             buffering=1024, size=None, block_transfer=False, force_segment=False):
+             buffering=1024, size=None, block_transfer=False, force_segment=False, request_crc_support=True):
         """Open the data stream as a file like object.
 
         :param int index:
@@ -185,14 +191,16 @@ class SdoClient(SdoBase):
             If block transfer should be used.
         :param bool force_segment:
             Force use of segmented download regardless of data size.
-
+        :param bool request_crc_support:
+            If crc calculation should be requested when using block transfer
+        
         :returns:
             A file like object.
         """
         buffer_size = buffering if buffering > 1 else io.DEFAULT_BUFFER_SIZE
         if "r" in mode:
             if block_transfer:
-                raw_stream = BlockUploadStream(self, index, subindex)
+                raw_stream = BlockUploadStream(self, index, subindex, request_crc_support=request_crc_support)
             else:
                 raw_stream = ReadableStream(self, index, subindex)
             if buffering:
@@ -201,7 +209,7 @@ class SdoClient(SdoBase):
                 return raw_stream
         if "w" in mode:
             if block_transfer:
-                raw_stream = BlockDownloadStream(self, index, subindex, size)
+                raw_stream = BlockDownloadStream(self, index, subindex, size, request_crc_support=request_crc_support)
             else:
                 raw_stream = WritableStream(self, index, subindex, size, force_segment)
             if buffering:
@@ -446,7 +454,7 @@ class BlockUploadStream(io.RawIOBase):
 
     crc_supported = False
 
-    def __init__(self, sdo_client, index, subindex=0):
+    def __init__(self, sdo_client, index, subindex=0, request_crc_support=True):
         """
         :param canopen.sdo.SdoClient sdo_client:
             The SDO client to use for reading.
@@ -454,6 +462,8 @@ class BlockUploadStream(io.RawIOBase):
             Object dictionary index to read from.
         :param int subindex:
             Object dictionary sub-index to read from.
+        :param bool request_crc_support:
+            If crc calculation should be requested when using block transfer            
         """
         self._done = False
         self.sdo_client = sdo_client
@@ -466,7 +476,9 @@ class BlockUploadStream(io.RawIOBase):
                      sdo_client.rx_cobid - 0x600)
         # Initiate Block Upload
         request = bytearray(8)
-        command = REQUEST_BLOCK_UPLOAD | INITIATE_BLOCK_TRANSFER | CRC_SUPPORTED
+        command = REQUEST_BLOCK_UPLOAD | INITIATE_BLOCK_TRANSFER
+        if request_crc_support:
+            command |= CRC_SUPPORTED
         struct.pack_into("<BHBBB", request, 0,
                          command, index, subindex, self.blksize, 0)
         response = sdo_client.request_response(request)
@@ -596,7 +608,7 @@ class BlockUploadStream(io.RawIOBase):
 class BlockDownloadStream(io.RawIOBase):
     """File like object for block download."""
 
-    def __init__(self, sdo_client, index, subindex=0, size=None):
+    def __init__(self, sdo_client, index, subindex=0, size=None, request_crc_support=True):
         """
         :param canopen.sdo.SdoClient sdo_client:
             The SDO client to use for communication.
@@ -606,6 +618,8 @@ class BlockDownloadStream(io.RawIOBase):
             Object dictionary sub-index to read from.
         :param int size:
             Size of data in number of bytes if known in advance.
+        :param bool request_crc_support:
+            If crc calculation should be requested when using block transfer            
         """
         self.sdo_client = sdo_client
         self.size = size
@@ -614,7 +628,11 @@ class BlockDownloadStream(io.RawIOBase):
         self._seqno = 0
         self._crc = sdo_client.crc_cls()
         self._last_bytes_sent = 0
-        command = REQUEST_BLOCK_DOWNLOAD | INITIATE_BLOCK_TRANSFER | CRC_SUPPORTED
+        self._current_block = []
+        self._retransmitting = False
+        command = REQUEST_BLOCK_DOWNLOAD | INITIATE_BLOCK_TRANSFER
+        if request_crc_support:
+            command |= CRC_SUPPORTED
         request = bytearray(8)
         logger.info("Initiating block download for 0x%X:%d", index, subindex)
         if size is not None:
@@ -692,7 +710,10 @@ class BlockDownloadStream(io.RawIOBase):
         request[1:len(b) + 1] = b
         self.sdo_client.send_request(request)
         self.pos += len(b)
-        if self.crc_supported:
+        # Add the sent data to the current block buffer
+        self._current_block.append(b)
+        # Don't calculate crc if retransmitting
+        if self.crc_supported and not self._retransmitting:
             # Calculate CRC
             self._crc.process(b)
         if self._seqno >= self._blksize:
@@ -715,14 +736,37 @@ class BlockDownloadStream(io.RawIOBase):
             raise SdoCommunicationError("Server did not respond with a "
                                         "block download response")
         if ackseq != self._blksize:
-            self.sdo_client.abort(0x05040003)
-            raise SdoCommunicationError(
-                ("%d of %d sequences were received. "
-                 "Retransmission is not supported yet.") % (ackseq, self._blksize))
+            # Sequence error, try to retransmit
+            self._retransmit(ackseq, blksize)
+            # We should be back in sync
+            return
+        # Clear the current block buffer
+        self._current_block = []
         logger.debug("All %d sequences were received successfully", ackseq)
         logger.debug("Server requested a block size of %d", blksize)
         self._blksize = blksize
         self._seqno = 0
+        
+    def _retransmit(self, ackseq, blksize):
+        """Retransmit the failed block"""
+        logger.info(("%d of %d sequences were received. "
+                 "Will start retransmission") % (ackseq, self._blksize))
+        # Sub blocks betwen ackseq and end of corrupted block need to be resent
+        # Get the part of the block to resend
+        block = self._current_block[ackseq:]
+        # Go back to correct position in stream
+        self.pos = self.pos - (len(block) * 7)
+        # Reset the _current_block before starting the retransmission
+        self._current_block = []
+        # Reset _seqno and update blksize
+        self._seqno = 0
+        self._blksize = blksize
+        # We are retransmitting
+        self._retransmitting = True
+        # Resend the block
+        for b in block:
+            self.write(b)
+        self._retransmitting = False
 
     def close(self):
         """Closes the stream."""
